/*  BLAZAR_SSC_ERC.c : One-zone homogeneous Syn/SSC/ERC code 
#                      for blazar SED fittings.
#  
#    Remark!   :  read 2 input files, 
#                   (1) Electron population  
#                   (2) Modified Bessel table : F_x_table_coarse        
#                 output 3 files,
#                   (3) Syn population in nu_Lnu
#                   (4) SSC population in nu_Lnu
#                   (5) ERC population in nu_Lnu 
#                   (6) Energy loss for both Sy and Comp scat.
#                       K_N effect is accurately considered
#                   (7) Blue Bump spectrum
#    
#    2007/4/25       version 2.0
#        
#                    Tokyo Tech     Jun  Kataoka        
*/

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

/*definition for array*/
#define MAX       2000
#define MAXF     20000
#define MAXSTR    1024
#define MAXCHR      64
#define BIN        200
#define BIN2       140    /* "bin width for SYNC" = "bin width for SSC" */
#define TBLMAX    1000     


/*definition for floating limit*/
#define EPS      1.000000e-5 
#define BRKLMT   1.000000e-100

/*definition for physical parameters*/
#define C1             4.19898e6
#define C2             1.865575e-23  
#define C3             1.02404e4  
#define NEGUNIT        -1.000000
#define UNIT           1.000000
#define PI             3.141592653589793
#define LIGHT          2.9979e10              /* in cm/s  */
#define LIGHT2         2.9979e5               /* in km/s  */
#define PLANCH         6.6261e-27             /* in erg*s */
#define RESTENE        8.1871e-7              /* in erg   */                
#define CORRECTION     0.7500000              /* geometrical hosei factor */
#define ELEC_RAD       2.8179e-13             /* in cm          */
#define THOMSON        6.65e-25               /* in cm^2        */
#define BOLTZ          1.38e-16               /* in erg deg^-1  */
#define SBSIG          5.6705e-5              /* erg/s/cm^2/K^4 */
#define OMG            0.270000               /* Omega_matter   */           
#define HUBBLE         71.0                   /* in Mpc/km/s    */
#define MPC            3.0857e24              /* in Mpc/km/s    */

#define CST1           0.1540
#define CST2           0.4304
#define CST3           0.19097
#define CST4           0.066941

#define PRINTLMT       1.0e-30
#define PRINTLMT2      1.0e30


/*definition of sub_functions*/
void LumiDis(double, double *);
void j_nu_a_nu(double, double, double, int, int, double *, 
               double *, double *,double *, double *, double *, double *);
void spheric_sync(double, double, double, double, double, double *, 
               double *,double *, double *, double *, double *, 
               double *, int *, double *, double *);
void spheric_SSC(double, double, double, double, double *, 
               double *, double *, double *,double *, int, int,  
               double *, double *, int *);
void spheric_ERC(double, double, double, double, double, double, double, 
               double, double, double, double *, double *, double *, 
               double *, double *,  int, int, double *,double *, 
               double *,double *, int *);
void comp_KN(double *, double *, double *, double *, double *, int, int, int);
void integ(int, double *, double *, double *);


main(int argc, char *argv[])
{
  double mag, radius, G_BLK, th_deg;             /* Input parmaters */    
  double redsft, Lbb, Liso, tau, extmp, lumidis; /* Input parmaters */    
  double beta, theta, delta;
  double nu_min, nu_Max;     
  double surface, b_fac;
  double gamma[MAX],N_gamma[MAX], gmj_up[MAX];
  double x[MAXF], F_x[MAXF];
  double freq[MAX], emis[MAX], abs[MAX];
  double nu_sync[MAX], nuLnu_sync[MAX];     
  double eps0[MAX], n_eps0[MAX], eps[MAX]; 
  double gm_dot_S[MAX], gm_dot_C[MAX];
  double nu_SSC[MAX], nuLnu_SSC[MAX];
  double nu_ERC[MAX], nuLnu_ERC[MAX];
  double nu_ext[MAX], fl_ext[MAX];
  double gm_ratio, gm_power;

  int    i,j,k,m,n;
  int    sum_colum, clm_comp;  
  int    column1, column2;
  char   temp1[MAXSTR], temp2[MAXSTR];
  char   temp3[MAXSTR], temp4[MAXSTR];
  char   line1[MAXSTR], line2[MAXSTR];
  char   file0[MAXCHR], file1[MAXCHR], file2[MAXCHR]; 
  char   file3[MAXCHR], file4[MAXCHR], file5[MAXCHR], file6[MAXCHR];
  FILE   *fp0, *fp1, *fp2, *fp3, *fp4, *fp5, *fp6; 

  printf("********** Input parameters for SSC/ERC calulation *********\n");
  printf("parameters: B, R, G_BLK, theta, z and frequency range are required!!\n");
  printf("\n");

  printf("Magnetic Field (Gauss)= ?  ");
  scanf("%lf",&mag);
  printf("Blob Radius (cm)= ?  ");
  scanf("%lf",&radius);
  printf("Bulk Lorentz factor= ?  ");
  scanf("%lf",&G_BLK);
  printf("Angle between observer to Jet (degree) ?  ");
  scanf("%lf",&th_deg);
  printf("Distance (redshift : z)= ?  ");
  scanf("%lf",&redsft);
  printf("Ext photons; Temp of Ext photons (deg) ?  ");
  scanf("%lf",&extmp);
  printf("Blue Bump photons; L_BB (erg/s) ?  ");
  scanf("%lf",&Lbb);
  printf("Conversion Factor of BB ? (tau < 1) ");
  scanf("%lf",&tau);
  printf("Synchrotron Freq. min value (Hz) = ?  ");
  scanf("%lf",&nu_min);
  printf("Synchrotron Freq. Max value (Hz) = ?  ");
  scanf("%lf",&nu_Max);
  printf("Open - Initial Electron population : N(gamma) -  Filename?");
  scanf("%s",file0);
  printf("Open - Modified Bessel function : F(x) -  Filename?"); 
  scanf("%s",file1);
  printf("Output - nu L_synch(nu) - Filename? (Obs frame)"); 
  scanf("%s",file2);
  printf("Output - nu L_SSC(nu)   - Filename? (Obs frame)"); 
  scanf("%s",file3);
  printf("Output - nu L_ERC(nu)   - Filename? (Obs frame)"); 
  scanf("%s",file4);
  printf("Output - Cooling coefficient?"); 
  scanf("%s",file5);
  printf("Output - Blue Bump Photons?"); 
  scanf("%s",file6);
  printf("\n");
  printf("********* Now Calculating ..... **************\n"); 
  printf("\n");

  if(NULL == (fp0 = fopen(file0,"r"))){
     printf("\n Cannot Open File : %s \n\n", file0);
     exit(1);
   }
  if(NULL == (fp1 = fopen(file1,"r"))){
     printf("\n Cannot Open File : %s \n\n", file1);
     exit(1);
   }
  if(NULL == (fp2 = fopen(file2, "w"))){
     printf("\n Cannot Open File : %s \n\n", file2);
     exit(1);
   }
  if(NULL == (fp3 = fopen(file3, "w"))){
     printf("\n Cannot Open File : %s \n\n", file3);
     exit(1);
   }
  if(NULL == (fp4 = fopen(file4, "w"))){
     printf("\n Cannot Open File : %s \n\n", file4);
     exit(1);
   }
  if(NULL == (fp5 = fopen(file5, "w"))){
     printf("\n Cannot Open File : %s \n\n", file5);
     exit(1);
   }
  if(NULL == (fp6 = fopen(file6, "w"))){
     printf("\n Cannot Open File : %s \n\n", file6);
     exit(1);
   }

  i = 0;
  while (fgets(line1, MAXSTR, fp0)!= NULL){
     if(sscanf(line1, "%s %s", temp1,temp2) != 2){ 
        printf("format error\n");
        exit(1);  
      }
     gamma[i]   = (double)atof(temp1);
     N_gamma[i] = (double)atof(temp2);
     i++;
   }
  column1 = (int) i;
  printf("electron column =%d\n",column1);
  fclose(fp0);

  j = 0;
  while (fgets(line2, MAXSTR, fp1)!= NULL){
     if(sscanf(line2, "%s %s", temp3,temp4) != 2){ 
        printf("format error\n");
        exit(1);  
      }
     x[j]   = (double)atof(temp3);
     F_x[j] = (double)atof(temp4);
     j++;
   }
  column2 = (int) j;
  printf("Bessel column =%d\n",column2);
  fclose(fp1);
  printf("\n");

  Liso = (double) Lbb * tau; 

/* calculating the beaming factor */
  theta = (double)th_deg * PI/180.0; 
  beta  = (double)sqrt(1.0 - 1.0/(G_BLK*G_BLK));
  delta = (double)1.0/(G_BLK * (1.0 - beta * cos(theta)));     
  printf("Doppler beaming factor is Delta = %f\n", delta);

/* Call Luminosity Distance              ..... lumidis            */
/*      Input  ...  redshift z                                    */
/*      Return ...  lumidis                                       */

  LumiDis(redsft, &lumidis);



/* Call emission/absorption function       ..... j_nu_a_nu        */
/*      Input  ...  gamma, N_gamma, x, F_x                        */
/*      Return ...  freq, emis, abs                               */

  j_nu_a_nu(mag, nu_min, nu_Max, column1, column2, gamma, N_gamma, 
                                           x, F_x, freq, emis, abs);


/* Call synchrotron function               ..... spheric_sync     */
/*      Input  ...  freq, emis, abs                               */
/*      Return ...  nu_sync, nuLnu_sync, eps0, n_eps0             */

  spheric_sync(lumidis, mag, radius, delta, redsft, freq, emis, abs, 
          nu_sync, nuLnu_sync, eps0, n_eps0, &sum_colum, &surface, &b_fac);


/* Call SSC function                       ..... spheric_SSC      */
/*      Input  ...  gamma, N_gamma,  eps0, n_eps0                 */
/*      Return ...  nu_SSC, nuLnu_SSC                             */

  spheric_SSC(radius, delta, surface, redsft, gamma, N_gamma, eps0, n_eps0, 
              eps, column1, sum_colum, nu_SSC, nuLnu_SSC,&clm_comp);

/* Call ERC function                       ..... spheric_ERC      */
/*      Input  ...  gamma, N_gamma,  eps0, n_eps0                 */
/*      Return ...  nu_ERC, nuLnu_ERC                             */

  spheric_ERC(lumidis, radius, theta, G_BLK, beta, delta, surface, redsft, 
             extmp, Liso, gamma, N_gamma, eps0, n_eps0, eps, column1, 
             sum_colum, nu_ERC, nuLnu_ERC, nu_ext, fl_ext, &clm_comp);

/* Call Compton cooling function in KN region ..... comp_KN       */
/*      Input  ...  eps_0, n_dps0, eps                            */
/*      Return ...  gm_dot_C                                      */

  comp_KN(gamma, eps0, n_eps0, eps, gm_dot_C, column1, sum_colum, clm_comp);

  for(k = 0; k < sum_colum; k++){
      fprintf(fp2, "%20.15e %20.15e\n",nu_sync[k], nuLnu_sync[k]);
  }
  for(k = sum_colum; k < BIN; k++){
      fprintf(fp2, "%20.15e %20.15e\n",nu_sync[k], PRINTLMT);
  }

  for(m = 0; m < BIN2; m++){
    if(nu_SSC[m] < PRINTLMT2){
      if(nuLnu_SSC[m] > PRINTLMT){
        fprintf(fp3, "%20.15e %20.15e\n",nu_SSC[m], nuLnu_SSC[m]); 
      }else{
        fprintf(fp3, "%20.15e %20.15e\n",nu_SSC[m], PRINTLMT);  
      }
    }
  }

  for(m = 0; m < BIN2; m++){
    if(nu_ERC[m] < PRINTLMT2){
      if(nuLnu_ERC[m] > PRINTLMT){
        fprintf(fp4, "%20.15e %20.15e\n",nu_ERC[m], nuLnu_ERC[m]); 
      }else{
        fprintf(fp4, "%20.15e %20.15e\n",nu_ERC[m], PRINTLMT); 
      }
    }
  }

  gm_ratio       = (double)gamma[column1-1]/gamma[0]; /*undefined variable*/
  gm_power       = (double)1.0/(column1-1.0);  
  gamma[column1] = (double)gamma[column1-1]*pow(gm_ratio,gm_power); 
  for(n=0; n <column1; n++){ 
    gmj_up[n]    = (double)sqrt(gamma[n]*gamma[n+1]);
    gm_dot_S[n]  = gmj_up[n] * gmj_up[n] * b_fac;
    fprintf(fp5, "%20.15e %20.15e %20.15e\n",gmj_up[n], gm_dot_S[n], 
                                                        gm_dot_C[n]);  
  }
  
  for(k=0;k<sum_colum-1;k++){
    if(fl_ext[k] > PRINTLMT){
      fprintf(fp6, "%20.15e %20.15e\n",nu_ext[k], fl_ext[k]/tau); 
    }
  }
}



void LumiDis(double redsft,double *lumidis)
{
  double dL, dA, R;
  double s,s2,s3, s4, ita1, ita2, a, a2, a3, a4;
  double tp2, tp3, tp4, tp5;

  s3 = (double)((1.0 - (double)OMG)/(double)OMG);
  s   = (double)pow((double)s3, 0.33333333);
  s2  = (double)s*s;
  s4  = (double)s3 * s ;
  
  tp2 = (double)sqrt((double)s3 + 1.0);
  tp3 = (double)(1.0 - (double)(CST1*s) + (double)(CST2*s2) 
                 + (double)(CST3*s3) + (double)(CST4*s4));
  ita1= (double)(2.0 * (double)tp2 / (double)pow(tp3, 0.125));
 
  a   = (double)(1.0/(1.0 + redsft));
  a2  = (double)a*a;
  a3  = (double)a*a*a;
  a4  = (double)a*a*a*a;
  tp5 = (double)((double)(1.0/a4) - (double)(CST1*s/a3) 
                 + (double)(CST2*s2/a2)
                 + (double)(CST3*s3/a) + (double)(CST4*s4));   
  ita2= (double)(2.0 * (double)tp2 / (double)pow(tp5, 0.125));


  dL  = (double)((double)(LIGHT2/HUBBLE) * (double)(1.0 + redsft) * 
                 (double)(ita1 - ita2));
  *lumidis = MPC * dL;

  /* size for 1" */
  dA  = (double)(dL/((1.0+redsft)*(1.0+redsft)));
  R   = (double)(dA * (PI/180.0) * (1/3600.0)); 

  printf("Luminosity Distance : d_L = %7.6f [Mpc]\n", dL);
  printf("corresponding to %7.6f [kpc/arcsec]\n", R*1000.0);
  printf("\n");
}



void j_nu_a_nu(double mag, double nu_min, double nu_Max, int column1, 
               int column2, double *gamma, double *N_gamma, double *x, 
               double *F_x, double *freq, double *emis, double *abs)
{
  double ratio, index; 
  double nu;
  double F_table;
  double inc[MAX];   
  double bla1[MAX],bla2[MAX],d_gamma[MAX]; 
  double comp_jnu[MAX], comp_anu[MAX];
  double j_nu, alpha_nu;
  int    k,p,q;  
  
  for(k=0; k<BIN; k++){
     ratio  = (double)(nu_Max/nu_min);
     index  = (double)k/BIN;
     nu     = (double)nu_min * pow(ratio,index);

     for(p=0; p<(column1-1); p++){
        inc[p]     = (double)nu/(C1*mag*gamma[p]*gamma[p]);
 
        for(q=0; q<(column2-1); q++){
           if((x[q]<inc[p]) && (x[q+1]>inc[p])){
              F_table = F_x[q];
              break;  
	   }else if(x[TBLMAX-1]<inc[p]){
              F_table = 0.0; 
              break; 
           } 
	 
	 }

      /*for emisson j_nu*/
        comp_jnu[p]   = (double)(N_gamma[p]*F_table);  
      /*for absorption alpha_nu*/
        bla1[p]      = (double)N_gamma[p]/(gamma[p]*gamma[p]);
        bla1[p+1]    = (double)N_gamma[p+1]/(gamma[p+1]*gamma[p+1]);
        d_gamma[p]   = (double)(gamma[p+1] - gamma[p]);  
        bla2[p]      = (double)(bla1[p+1] - bla1[p])/d_gamma[p];  
        comp_anu[p]  = (double)gamma[p]*gamma[p]*bla2[p] * F_table; 
    }
     integ(column1, gamma, comp_jnu, &j_nu);   
     integ(column1, gamma, comp_anu, &alpha_nu);   

     freq[k]         = (double)nu;  
     emis[k]         = (double)j_nu * mag * C2;  
     abs[k]          = (double)alpha_nu * NEGUNIT * mag * C3/ (nu*nu);
     abs[k]          = (double)fabs(abs[k]);
     emis[k]         = (double)fabs(emis[k]);
   }  
}


void spheric_sync(double lumidis, double mag, double radius, double delta, 
                  double redsft, double *freq, double *emis, double *abs, 
                  double *obs_nu, double *obs_L, double *eps0, double *n_eps0,
                  int *sum_colum, double *surface, double *b_fac)
{
  double index,index2;
  double fac1,fac2,fac3;
  double factor1,factor2,factor3;
  double comp1,comp2,comp3,comp4,comp5;
  double comp6,comp7,comp8,comp9,comp10;
  double lumi_tot;
  double u_B, u_sync, u_sync2;
  double z_hosei;
  double bunsi, bunbo;
  double L_nu[MAX], nuL_nu[MAX], ene_dens[MAX];    
  int i,j,k,p,q;  
  int colm1, colm2;

  for (i=0;i<BIN; i++){
    if((0.0<emis[i]) && (emis[i]<BRKLMT)){
        break;
    }else if (emis[i] == 0.0){
        ;
    }
  }
  colm1 = i; 
  printf("j_nu column =%d\n",colm1);

  for (j=0;j<BIN; j++){
    if((0.0<abs[j]) && (abs[j]<BRKLMT)){
      break;
    }else if (abs[j] == 0.0){
      ;
    }   
  }
  colm2 = j; 
  printf("alpha_nu column =%d\n",colm2);

  if(colm1 > colm2){
    *sum_colum = colm2;       
  } else {
    *sum_colum = colm1;
  } 

  if(colm1 != colm2){
     printf("No. of j_nu and alpha_nu are different!\n"); 
     printf("We set sum column No to %d...\n", *sum_colum); 
  }

  for(k=0; k<(*sum_colum); k++){
    if(abs[k] != 0.0){
       index     = (double)(2.0*abs[k]*radius);
       fac1      = (double)(4.0*PI*PI*radius*radius);
       fac2      = (double)(emis[k]/abs[k]);

       if(index>EPS){
          comp1     = (double)exp(NEGUNIT*index);
          comp2     = (double)(UNIT - comp1*(index+1.0));
          comp3     = (double)2.0/(index*index);
          fac3      = (double)(UNIT - comp3*comp2);
       } else {
          comp4     = (double)(2.0/3.0)*index;
          comp5     = (double)pow(index,2.0)/4.0;
          fac3      = (double)comp4-comp5;
       }
          L_nu[k]   = (double)fac1*fac2*fac3;
          nuL_nu[k] = (double)freq[k]*L_nu[k];

          *surface  = (double)4.0 * PI * lumidis * lumidis;

          z_hosei   = (double)(1.0 + redsft);
          obs_nu[k] = (double)freq[k]*delta/z_hosei;
        
          bunsi     = (double)nuL_nu[k]*pow(delta,4.0);
          bunbo     = (double)(*surface);
          obs_L[k]  = (double)bunsi/bunbo;

    }else{   /* to avoid floating error */
          L_nu[k]   =  0.0; 
          nuL_nu[k] =  0.0;
          z_hosei   =  (double)(1.0 + redsft);
          obs_nu[k] =  (double)freq[k]*delta/z_hosei; 
          obs_L[k]  =  0.0;
          *surface  =  (double)4.0 * PI * lumidis * lumidis;
    }
  }
  for(k=*sum_colum;k<BIN;k++){
          obs_nu[k] =  (double)freq[k]*delta/z_hosei; 
  }

     integ(*sum_colum, freq, L_nu, &lumi_tot);      

     printf("\n");
     printf("Total Synchrotorn Luminosity in the source frame is\n");
     printf("L_synch = %e (erg/s)\n", lumi_tot);

/*   u_sync   =  (double)lumi_tot/(4*PI*radius*radius*LIGHT); */
     u_B      =  (double)mag*mag/(8.0*PI);
     *b_fac   =  (double)4.0* u_B * THOMSON * LIGHT/(3.0 * RESTENE);   

/*  printf("u_sync1 = %e (erg/cm^3)\n", u_sync); */
  printf("u_B     = %e (erg/cm^3)\n", u_B);
  printf("b_fac   = %e\n", *b_fac);

  for(p=0; p<*sum_colum; p++){
     eps0[p]   = (double)(PLANCH * freq[p])/RESTENE;     
     index2    = (double)abs[p]*radius;
     factor1   = (double)4.0*PI;
     factor2   = (double)UNIT/(PLANCH*LIGHT*eps0[p]);

     if(index2>EPS){
     }else {
        comp7     = (double)index2;
        comp8     = (double)pow(index2,2.0)/2.0;
        comp9     = (double)pow(index2,3.0)/6.0;
        comp10    = (double)pow(index2,4.0)/24.0;
        factor3   = (double)(comp7-comp8+comp9-comp10);
      }
      
    
     if(abs[p] != 0.0){
        n_eps0[p] = (double)factor1*factor2*CORRECTION*
                               (emis[p]/abs[p])*factor3;
     }else{               /* to avoid floating error! */
        n_eps0[p] = 0.0;
     }
     ene_dens[p]     = (double)eps0[p] * n_eps0[p] * RESTENE;       
  }
  integ((*sum_colum-1), eps0, ene_dens, &u_sync2);      
  printf("u_sync  = %e (erg/cm^3)\n", u_sync2);
  printf("\n");
}


void spheric_SSC(double radius, double delta, double surface, double redsft, 
                 double *gamma, double *N_gamma, double *eps0, double *n_eps0,
                 double *eps, int clm_elec, int clm_sync, double *nu_SSC, 
                 double *nuLnu_SSC, int *clm_comp)
{
  double  d_gamma[MAX], d_eps0[MAX];
  double  gamma_max;
  double  nu_SSC_rest[MAX];
  double  eps0_min;
  double  bunsi, bunbo;
  double  z_hosei;
  double  q_eps[MAX];
  double  j_nu_SSC[MAX], L_nu_SSC[MAX];
  double  nuLnu_SSC_rest[MAX];

  double  lumi_tot; 
  double  lower, upper;  
  double  factor;
  double  kappa,fac;
  double  term1,term2,term3;
  double  term3_u,term3_d; 
  double  index;
  double  ratio;
  double  section;
  static  double  sum1,sum2;
  double  chk_up, chk_low;
  int     p,q,r;
  
  sum1 = 0;       
  sum2 = 0;
  gamma_max  = gamma[clm_elec-1];
  eps0_min   = eps0[0];
  *clm_comp  = BIN2;

  for(p=0;p<*clm_comp;p++){          
     for(q=0;q<clm_sync-1;q++){
        for(r=0;r<clm_elec-1;r++){
           d_gamma[r]    =  (double)gamma[r+1] - gamma[r] ;
           d_eps0[q]     =  (double)eps0[q+1] - eps0[q] ;

           lower         =  (double)eps0_min ; 
           upper         =  (double)gamma_max;
           ratio         =  (double)upper/lower;   
           index         =  (double)(p+1)/BIN2;
           eps[p]        =  (double)lower*pow(ratio,index);

           chk_low       =  (double)eps0[q];
           chk_up        =  (double)(4.0 * gamma[r]*gamma[r]*eps0[q])
                                          /(1.0+4.0*eps0[q]*gamma[r]);

           if((chk_low<eps[p]) && (eps[p]<chk_up)){      
              kappa         =  (double)eps[p]/(4*eps0[q]*gamma[r]
                                                      *(gamma[r]-eps[p]));
              factor        =  (double)2.0*PI*ELEC_RAD*ELEC_RAD*LIGHT
                                       /(gamma[r]*gamma[r]*eps0[q]);
              term1         =  (double)2.0*kappa*log(kappa);
              term2         =  (double)(1.0+2.0*kappa)*(1.0-kappa);
              fac           =  (double)4.0*eps0[q]*gamma[r]*kappa; 
              term3_u       =  (double)fac *fac *(1.0-kappa);
              term3_d       =  (double)2.0*(1.0+4.0*eps0[q]*gamma[r]*kappa);   
              term3         =  (double)term3_u/term3_d;
              section       =  (double)factor*(term1  + term2 + term3); 
              sum1          =   sum1 + (double)N_gamma[r]*d_gamma[r]
                                                 *section*n_eps0[q]*d_eps0[q];
          } else {
            sum1 = sum1;
	  }
        }
          sum2 = sum2 + sum1;
          sum1 = 0.0;      
       }  

       q_eps[p] = sum2;

       j_nu_SSC[p]      = (double)PLANCH*eps[p]*q_eps[p]/(4.0*PI);
       L_nu_SSC[p]      = (double)(16.0*PI*PI/3.0)*pow(radius,3.0)*j_nu_SSC[p];
       nu_SSC_rest[p]   = (double)RESTENE*eps[p]/PLANCH;
       nuLnu_SSC_rest[p]= (double)nu_SSC_rest[p]*L_nu_SSC[p]; 
       z_hosei          = (double)1.0 + redsft;
       nu_SSC[p]        = (double)nu_SSC_rest[p]*delta/z_hosei;
       bunsi            = (double)nuLnu_SSC_rest[p]*pow(delta,4.0);
       bunbo            = (double)surface;
       nuLnu_SSC[p]     = (double)bunsi/bunbo;  
       sum2 = 0.0;
      }       
     integ(BIN2, nu_SSC_rest, L_nu_SSC, &lumi_tot);      
     printf("Total SSC Luminosity in the source frame is\n");
     printf("L_SSC = %e (erg/s)\n ", lumi_tot);
     printf("\n");
}

void spheric_ERC(double lumidis, double radius, double theta, double G_BLK, 
               double beta, double delta, double surface, double redsft, 
               double extmp, double Liso, double *gamma, double *N_gamma, 
               double *eps0, double *n_eps0, double *eps, int clm_elec, 
	       int clm_sync, double *nu_ERC, double *nuLnu_ERC, double *nu, 
               double *fl, int *clm_comp)
{
  double  d_gamma[MAX], d_eps0[MAX];
  double  gamma_max;
  double  nu_ERC_rest[MAX];
  double  eps0_min;
  double  bunsi, bunbo;
  double  q_eps[MAX];
  double  j_nu_ERC[MAX], L_nu_ERC[MAX];
  double  nuLnu_ERC_rest[MAX];

  double  lumi_tot; 
  double  lower, upper;  
  double  factor;
  double  kappa,fac, z_hosei;
  double  term1,term2,term3;
  double  term3_u,term3_d; 
  double  index;
  double  ratio, ang_rat;
  double  section;
  double  bafac1, bafac2, bafac3, bafac4, bafac5;
  static  double  sum1,sum2;
  double  chk_up, chk_low;
  double  temp0, temp1, temp2, temp3, temp4, temp5;
  int     p,q,r,ba;
  
  sum1 = 0;       
  sum2 = 0;
  gamma_max  = gamma[clm_elec-1];
  eps0_min   = eps0[0];
  *clm_comp  = BIN2;
  z_hosei   = (double)1.0 + redsft;

  /* T_ext(jet) = T_ext(iso)*G_BLK & Norm \propt T^4 \propt G_BLK^4*/
  for(ba=0;ba<clm_sync-1;ba++){
     bafac1 = (double)eps0[ba]*eps0[ba]/G_BLK/G_BLK;  
     bafac2 = (double)(RESTENE/PLANCH/LIGHT)*
                         (RESTENE/PLANCH/LIGHT)*
                                (RESTENE/PLANCH/LIGHT);
     bafac3 = (double)exp(eps0[ba]*RESTENE/(BOLTZ*extmp*G_BLK)) - 1.0;
     bafac4 = (double)PI/(SBSIG*extmp*extmp*extmp*extmp);
     bafac5 = (double)Liso/(radius*radius);
     n_eps0[ba] = bafac1 * bafac2 * bafac4 * bafac5 / bafac3; 
   }

  for(ba=0;ba<clm_sync-1;ba++){
     temp0 = (double)(eps0[ba]*RESTENE)/PLANCH;
     temp1 = (double)2.0 * PLANCH * temp0*temp0*temp0/(LIGHT*LIGHT);
     temp2 = (double)exp(eps0[ba]*RESTENE/(BOLTZ*extmp)) - 1.0;     
     temp3 = (double)(SBSIG/PI) * extmp * extmp * extmp * extmp;     
     temp4 = (double)Liso/(4.0*PI*lumidis*lumidis);     
     temp5 = (temp0 * temp1* temp4)/ (temp2 * temp3);  
     nu[ba]= (double) temp0;
     fl[ba]= (double) temp5; 
   }

  for(p=0;p<*clm_comp;p++){          
     for(q=0;q<clm_sync-1;q++){
        for(r=0;r<clm_elec-1;r++){
           d_gamma[r]    =  (double)gamma[r+1] - gamma[r] ;
           d_eps0[q]     =  (double)eps0[q+1] - eps0[q] ;

           lower         =  (double)eps0_min ; 
           upper         =  (double)gamma_max;
           ratio         =  (double)upper/lower;   
           index         =  (double)(p+1)/BIN2;
           eps[p]        =  (double)lower*pow(ratio,index);

           chk_low       =  (double)eps0[q];
           chk_up        =  (double)(4.0 * gamma[r]*gamma[r]*eps0[q])
                                          /(1.0+4.0*eps0[q]*gamma[r]);

           if((chk_low<eps[p]) && (eps[p]<chk_up)){      
              kappa         =  (double)eps[p]/(4*eps0[q]*gamma[r]
                                                      *(gamma[r]-eps[p]));
              factor        =  (double)2.0*PI*ELEC_RAD*ELEC_RAD*LIGHT
                                       /(gamma[r]*gamma[r]*eps0[q]);
              term1         =  (double)2.0*kappa*log(kappa);
              term2         =  (double)(1.0+2.0*kappa)*(1.0-kappa);
              fac           =  (double)4.0*eps0[q]*gamma[r]*kappa; 
              term3_u       =  (double)fac *fac *(1.0-kappa);
              term3_d       =  (double)2.0*(1.0+4.0*eps0[q]*gamma[r]*kappa);   
              term3         =  (double)term3_u/term3_d;
              section       =  (double)factor*(term1  + term2 + term3); 
              sum1          =   sum1 + (double)N_gamma[r]*d_gamma[r]
                                                 *section*n_eps0[q]*d_eps0[q];
          } else {
            sum1 = sum1;
	  }
        }
          sum2 = sum2 + sum1;
          sum1 = 0.0;      
       }  

       q_eps[p] = sum2;

       j_nu_ERC[p]      = (double)PLANCH*eps[p]*q_eps[p]/(4.0*PI);
       L_nu_ERC[p]      = (double)(16.0*PI*PI/3.0)*pow(radius,3.0)*j_nu_ERC[p];
       nu_ERC_rest[p]   = (double)RESTENE*eps[p]/PLANCH;
       nuLnu_ERC_rest[p]= (double)nu_ERC_rest[p]*L_nu_ERC[p];
       nu_ERC[p]        = (double)nu_ERC_rest[p]*delta/z_hosei;
       bunsi            = (double)nuLnu_ERC_rest[p]*pow(delta,6.0);
       bunbo            = (double)surface;
       ang_rat          = (double)(1.0 + cos(theta))/(1.0 + beta);
       nuLnu_ERC[p]     = (double)bunsi* pow(ang_rat, 2.0)
                                         /(bunbo * pow(G_BLK,2.0));
       sum2 = 0.0;
      }       
     integ(BIN2, nu_ERC_rest, L_nu_ERC, &lumi_tot);      
     printf("Total ERC Luminosity in the source frame is\n");
     printf("L_ERC = %e (erg/s)\n ", lumi_tot);
     printf("\n");
}

void comp_KN(double *gamma, double *eps0, double *n_eps0, double *eps, 
             double *gm_dot_C, int clm_elec, int clm_sync, int clm_comp)
{
  double  gmj_up[MAX];
  double  d_eps0[MAX], d_eps[MAX];
  double  eps0_min;
  double  gm_ratio, gm_power;
  double  factor;
  double  kappa,fac;
  double  term1,term2,term3;
  double  term3_u,term3_d; 
  double  index;
  double  ratio;
  double  section;
  static  double  sum1, sum2;
  double  chk_up, chk_low;
  int     p,q,r;
  
  sum1 = 0;       
  sum2 = 0;
  gm_ratio         = (double)gamma[clm_elec-1]/gamma[0]; /*undefined variable*/
  gm_power         = (double)1.0/(clm_elec-1.0);  
  gamma[clm_elec]  = (double)gamma[clm_elec-1]*pow(gm_ratio,gm_power); 

  for(p=0;p<clm_elec;p++){          
     gmj_up[p]  = (double)sqrt(gamma[p]*gamma[p+1]);
     for(q=0;q<clm_sync-1;q++){
        for(r=0;r<clm_comp-1;r++){
           d_eps[r]      =  (double)eps[r+1] - eps[r] ;
           d_eps0[q]     =  (double)eps0[q+1] - eps0[q] ;
           chk_low       =  (double)eps0[q];
           chk_up        =  (double)(4.0 * gmj_up[p]*gmj_up[p]*eps0[q])
                                          /(1.0+4.0*eps0[q]*gmj_up[p]);

           if((chk_low<eps[r]) && (eps[r]<chk_up)){      
              kappa         =  (double)eps[r]/(4*eps0[q]*gmj_up[p]
                                                      *(gmj_up[p]-eps[r]));
              factor        =  (double)2.0*PI*ELEC_RAD*ELEC_RAD*LIGHT
                                       /(gmj_up[p]*gmj_up[p]*eps0[q]);
              term1         =  (double)2.0*kappa*log(kappa);
              term2         =  (double)(1.0+2.0*kappa)*(1.0-kappa);
              fac           =  (double)4.0*eps0[q]*gmj_up[p]*kappa; 
              term3_u       =  (double)fac *fac *(1.0-kappa);
              term3_d       =  (double)2.0*(1.0+4.0*eps0[q]*gmj_up[p]*kappa);   
              term3         =  (double)term3_u/term3_d;
              section       =  (double)factor*(term1  + term2 + term3); 
              sum1          =   sum1 + (double)section*n_eps0[q]*d_eps0[q]
                                               *eps[r]*d_eps[r];
          } else {
            sum1 = sum1;
	  }
	 }
          sum2 = sum2 + sum1;
          sum1 = 0.0;      
       }  
       gm_dot_C[p] = sum2;
       sum2 = 0.0;
      }       
}


void integ(int clm, double *gma, double *cmp, double *ret)
{
  double sum_K;
  double au1,au2,ad1;
  double bu1,bu2,bu3,bu4,bd1,bd2,bd3;
  double s1,s2,s3;
  double a,b,c;
  double s[MAX];
  int pare;
  int r,u;


  if ((clm%2) == 0){
    pare  = (int)(clm/2 - 1);
  } else { 
    pare = (int)(clm/2);
  } 
  sum_K = 0.0;
  for(r=0;r<pare; r++){            
      u = (int)2*r;    
      bu1 = (double)(gma[u+1]*gma[u+1] - gma[u+2]*gma[u+2]);
      bu2 = (double)(cmp[u] - cmp[u+1]); 
      bu3 = (double)(gma[u]*gma[u] - gma[u+1]*gma[u+1]);            
      bu4 = (double)(cmp[u+1] - cmp[u+2]);
      bd1 = (double)(gma[u] - gma[u+1]);
      bd2 = (double)(gma[u+1] - gma[u+2]);
      bd3 = (double)(gma[u+2] - gma[u]);
      b   = (double)(bu1*bu2 -bu3*bu4)/(bd1*bd2*bd3) ;
 
      au1 = bu2;
      au2 = bd1;
      ad1 = bu3;
      a   = (double)(au1-b*au2)/ad1;

      c   = (double)cmp[u] - a*gma[u]*gma[u] -b*gma[u];

      s1  = (double)(a/3.0)* (gma[u+2]*gma[u+2]*gma[u+2] 
                                       - gma[u]*gma[u]*gma[u]);  
      s2  = (double)(b/2.0)* (gma[u+2]*gma[u+2] - gma[u]*gma[u]);  
      s3  = (double) c *(gma[u+2] - gma[u]);
      s[r] =(double)(s1+s2+s3);  
      sum_K = sum_K + s[r];     
    }
    if ((clm%2) == 0){
       sum_K = sum_K + (gma[clm-1] - gma[clm-2]) * cmp[clm-2];
    }else{
       ; 
    } 
    *ret = (double)sum_K;
}



